# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class SourceRepository(Model):
    """SourceRepository.

    :param identifier:
    :type identifier: str
    :param properties: The properties cannot be IEnumerable/IList, since XML
     serializer does not work well with interfaces. This object gets passed as
     job parameter which uses XML serializer.
    :type properties: list of :class:`Property <azuretfs.models.Property>`
    :param repository_type:
    :type repository_type: object
    """

    _attribute_map = {
        'identifier': {'key': 'identifier', 'type': 'str'},
        'properties': {'key': 'properties', 'type': '[Property]'},
        'repository_type': {'key': 'repositoryType', 'type': 'object'},
    }

    def __init__(self, identifier=None, properties=None, repository_type=None):
        self.identifier = identifier
        self.properties = properties
        self.repository_type = repository_type



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class SourceConfiguration(Model):
    """SourceConfiguration.

    :param source_repository:
    :type source_repository: :class:`SourceRepository
     <azuretfs.models.SourceRepository>`
    :param source_version:
    :type source_version: str
    """

    _attribute_map = {
        'source_repository': {'key': 'sourceRepository', 'type': 'SourceRepository'},
        'source_version': {'key': 'sourceVersion', 'type': 'str'},
    }

    def __init__(self, source_repository=None, source_version=None):
        self.source_repository = source_repository
        self.source_version = source_version



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RepositoryInfo(Model):
    """RepositoryInfo.

    :param id:
    :type id: str
    :param name:
    :type name: str
    :param url:
    :type url: str
    :param remote_url:
    :type remote_url: str
    :param project_info:
    :type project_info: TeamProjectInfo
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'remote_url': {'key': 'remoteUrl', 'type': 'str'},
        'project_info': {'key': 'project', 'type': 'TeamProjectInfo'},
    }

    def __init__(self, id=None, name=None, url=None, remote_url=None, project_info=None):
        self.id = id
        self.name = name
        self.url = url
        self.remote_url = remote_url
        self.project_info = project_info



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from .property import Property
from .resource_configuration import ResourceConfiguration
from .pipeline_configuration import PipelineConfiguration
from .source_repository import SourceRepository
from .source_configuration import SourceConfiguration
from .continuous_deployment_configuration import ContinuousDeploymentConfiguration
from .continuous_deployment_operation import ContinuousDeploymentOperation
from .reference_links import ReferenceLinks
from .operation import Operation
from .operation_reference import OperationReference
from .collection_info import CollectionInfo
from .repository_info import RepositoryInfo
from .team_project_info import TeamProjectInfo
from .vsts_info import VstsInfo

__all__ = [
    'Property',
    'ResourceConfiguration',
    'PipelineConfiguration',
    'SourceRepository',
    'SourceConfiguration',
    'ContinuousDeploymentConfiguration',
    'ContinuousDeploymentOperation',
    'ReferenceLinks',
    'Operation',
    'OperationReference',
    'CollectionInfo',
    'RepositoryInfo',
    'TeamProjectInfo',
    'VstsInfo',
]



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ContinuousDeploymentConfiguration(Model):
    """ContinuousDeploymentConfiguration.

    :param account_configuration:
    :type account_configuration: :class:`ResourceConfiguration
     <azuretfs.models.ResourceConfiguration>`
    :param pipeline_configuration:
    :type pipeline_configuration: :class:`PipelineConfiguration
     <azuretfs.models.PipelineConfiguration>`
    :param project_configuration:
    :type project_configuration: :class:`ResourceConfiguration
     <azuretfs.models.ResourceConfiguration>`
    :param source_configuration:
    :type source_configuration: :class:`SourceConfiguration
     <azuretfs.models.SourceConfiguration>`
    :param target_configuration:
    :type target_configuration: list of :class:`Property
     <azuretfs.models.Property>`
    :param test_configuration:
    :type test_configuration: list of :class:`Property
     <azuretfs.models.Property>`
    """

    _attribute_map = {
        'account_configuration': {'key': 'accountConfiguration', 'type': 'ResourceConfiguration'},
        'pipeline_configuration': {'key': 'pipelineConfiguration', 'type': 'PipelineConfiguration'},
        'project_configuration': {'key': 'projectConfiguration', 'type': 'ResourceConfiguration'},
        'source_configuration': {'key': 'sourceConfiguration', 'type': 'SourceConfiguration'},
        'target_configuration': {'key': 'targetConfiguration', 'type': '[Property]'},
        'test_configuration': {'key': 'testConfiguration', 'type': '[Property]'},
    }

    def __init__(self, account_configuration=None, pipeline_configuration=None, project_configuration=None, source_configuration=None, target_configuration=None, test_configuration=None):
        self.account_configuration = account_configuration
        self.pipeline_configuration = pipeline_configuration
        self.project_configuration = project_configuration
        self.source_configuration = source_configuration
        self.target_configuration = target_configuration
        self.test_configuration = test_configuration



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ResourceConfiguration(Model):
    """ResourceConfiguration.

    :param create_if_not_found:
    :type create_if_not_found: bool
    :param properties:
    :type properties: list of :class:`Property <azuretfs.models.Property>`
    :param resource_name:
    :type resource_name: str
    """

    _attribute_map = {
        'create_if_not_found': {'key': 'createIfNotFound', 'type': 'bool'},
        'properties': {'key': 'properties', 'type': '[Property]'},
        'resource_name': {'key': 'resourceName', 'type': 'str'},
    }

    def __init__(self, create_if_not_found=None, properties=None, resource_name=None):
        self.create_if_not_found = create_if_not_found
        self.properties = properties
        self.resource_name = resource_name



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import re
import time

try:
    from urllib.parse import quote, urlparse
except ImportError:
    from urllib import quote  #pylint: disable=no-name-in-module
    from urlparse import urlparse  #pylint: disable=import-error
from azuretfs import AzureTfs, VstsInfoProvider
from azuretfs.models import (ContinuousDeploymentConfiguration, ResourceConfiguration,
                             SourceConfiguration, SourceRepository, Property, VstsInfo)

# Use this class to setup or remove continuous delivery mechanisms for Azure web sites using VSTS build and release
class ContinuousDeliveryManager(object):
    def __init__(self, progress_callback):
        """
        Use this class to setup or remove continuous delivery mechanisms for Azure web sites using VSTS build and release
        :param progress_callback: method of the form func(count, total, message)
        """
        self._update_progress = progress_callback or self._skip_update_progress
        self._azure_info = _AzureInfo()
        self._repo_info = _RepositoryInfo()

    def get_vsts_app_id(self):
        """
        Use this method to get the 'resource' value for creating an Azure token to be used by VSTS
        :return: App id for VSTS
        """
        return '499b84ac-1321-427f-aa17-267ca6975798'

    def set_azure_web_info(self, resource_group_name, website_name, credentials,
                           subscription_id, subscription_name, tenant_id, webapp_location):
        """
        Call this method before attempting to setup continuous delivery to setup the azure settings
        :param resource_group_name:
        :param website_name:
        :param credentials:
        :param subscription_id:
        :param subscription_name:
        :param tenant_id:
        :param webapp_location:
        :return:
        """
        self._azure_info.resource_group_name = resource_group_name
        self._azure_info.website_name = website_name
        self._azure_info.credentials = credentials
        self._azure_info.subscription_id = subscription_id
        self._azure_info.subscription_name = subscription_name
        self._azure_info.tenant_id = tenant_id
        self._azure_info.webapp_location = webapp_location

    def set_repository_info(self, repo_url, branch, git_token):
        """
        Call this method before attempting to setup continuous delivery to setup the source control settings
        :param repo_url:
        :param branch:
        :param git_token:
        :return:
        """
        self._repo_info.url = repo_url
        self._repo_info.branch = branch
        self._repo_info.git_token = git_token

    def remove_continuous_delivery(self):
        """
        To be Implemented
        :return:
        """
        # TODO: this would be called by appservice web source-control delete
        return

    def setup_continuous_delivery(self, azure_deployment_slot, app_type, vsts_account_name, create_account,
                                  vsts_app_auth_token):
        """
        Use this method to setup Continuous Delivery of an Azure web site from a source control repository.
        :param azure_deployment_slot: the slot to use for deployment
        :param app_type: the type of app that will be deployed. i.e. AspNetWap, AspNetCore, etc.
        :param vsts_account_name:
        :param create_account:
        :param vsts_app_auth_token:
        :return: a message indicating final status and instructions for the user
        """

        app_type = self._get_app_project_type(app_type)
        branch = self._repo_info.branch or 'refs/heads/master'

        # Verify inputs before we start generating tokens
        sourceRepository, account_name, team_project_name = self._get_source_repository(self._repo_info.url,
            self._repo_info.git_token, self._azure_info.credentials)
        self._verify_vsts_parameters(vsts_account_name, sourceRepository)
        vsts_account_name = vsts_account_name or account_name
        cd_project_name = team_project_name or self._azure_info.website_name

        # Create AzureTfs client
        az_tfs = AzureTfs('3.2-preview', None, self._azure_info.credentials)

        # Construct the config body of the continuous delivery call
        account_configuration = ResourceConfiguration(create_account,
                                                      [Property('region', 'CUS'),
                                                       Property('PortalExtensionUsesNewAcquisitionFlows', 'false')],
                                                      vsts_account_name)
        pipeline_configuration = None  # This is not set because we are not the ibiza portal
        project_configuration = ResourceConfiguration(True, [Property('','')], cd_project_name)
        source_configuration = SourceConfiguration(sourceRepository, branch)
        target_configuration = [Property('resourceProperties', app_type),
                                Property('resourceGroup', self._azure_info.resource_group_name),
                                Property('subscriptionId', self._azure_info.subscription_id),
                                Property('subscriptionName', self._azure_info.subscription_name),
                                Property('tenantId', self._azure_info.tenant_id),
                                Property('resourceName', self._azure_info.website_name),
                                Property('deploymentSlot', azure_deployment_slot),
                                Property('location', self._azure_info.webapp_location),
                                Property('AuthInfo', 'Bearer ' + vsts_app_auth_token)]
        test_configuration = [Property('appServicePlan',''),
                              Property('appServicePlanName',''),
                              Property('appServicePricingTier','Premium'),
                              Property('testWebAppLocation', self._azure_info.webapp_location),
                              Property('testWebAppName', self._azure_info.website_name)]
        config = ContinuousDeploymentConfiguration(account_configuration, pipeline_configuration, project_configuration,
                                                   source_configuration, target_configuration, test_configuration)
        # Configure the continuous deliver using VSTS as a backend
        response = az_tfs.configure_continuous_deployment(config)
        if response.status == 'inProgress':
            final_status = self._wait_for_cd_completion(az_tfs, response)
            return self._get_summary(final_status, vsts_account_name, self._azure_info.subscription_id, self._azure_info.resource_group_name, self._azure_info.website_name)
        else:
            raise RuntimeError('Unknown status returned from configure_continuous_deployment: ' + response.status)

    def _verify_vsts_parameters(self, cd_account, sourceRepository):
        # if provider is vsts and repo is not vsts then we need the account name
        if sourceRepository.repository_type in [2, 4] and not cd_account:
            raise RuntimeError('You must provide a value for cd-account since your repo-url is not a VSTS repository.')

    def _get_source_repository(self, uri, token, cred):
        # Determine the type of repository (vstsgit == 1, github == 2, tfvc == 3, externalGit == 4)
        # Find the identifier and set the properties; default to externalGit
        type = 4
        identifier = uri
        properties = []
        account_name = None
        team_project_name = None
        match = re.match(r'[htps]+\:\/\/(.+)\.visualstudio\.com.*\/_git\/(.+)', uri, re.IGNORECASE)
        if match:
            type = 1
            account_name = match.group(1)
            # we have to get the repo id as the identifier
            info = self._get_vsts_info(uri, cred)
            identifier = info.repository_info.id
            team_project_name = info.repository_info.project_info.name
        else:
            match = re.match(r'[htps]+\:\/\/github\.com\/(.+)', uri, re.IGNORECASE)
            if match:
                type = 2
                identifier = match.group(1)
                properties = [Property('accessToken', token)]
            else:
                match = re.match(r'[htps]+\:\/\/(.+)\.visualstudio\.com\/(.+)', uri, re.IGNORECASE)
                if match:
                    type = 3
                    identifier = match.group(2)
                    account_name = match.group(1)
        sourceRepository = SourceRepository(identifier, properties, type)
        return sourceRepository, account_name, team_project_name

    def _get_vsts_info(self, vsts_repo_url, cred):
        vsts_info_client = VstsInfoProvider('3.2-preview', vsts_repo_url, cred)
        return vsts_info_client.get_vsts_info()

    def _get_app_project_type(self, cd_app_type):
        app_type =  '{{\"webAppProjectType\":\"{}\"}}'.format(cd_app_type)
        return app_type

    def _wait_for_cd_completion(self, az_tfs, response):
        # Wait for the configuration to finish and report on the status
        step = 5
        max = 100
        self._update_progress(step, max, 'Setting up VSTS continuous deployment')
        status = az_tfs.get_continuous_deployment_operation(response.id)
        while status.status == 'queued' or status.status == 'inProgress':
            step += 5 if step + 5 < max else 0
            self._update_progress(step, max, 'Setting up VSTS continuous deployment (' + status.status + ')')
            time.sleep(2)
            status = az_tfs.get_continuous_deployment_operation(response.id)
        if status.status == 'failed':
            self._update_progress(max, max, 'Setting up VSTS continuous deployment (FAILED)')
            raise RuntimeError(status.result_message)
        self._update_progress(max, max, 'Setting up VSTS continuous deployment (SUCCEEDED)')
        return status

    def _get_summary(self, final_status, account_name, subscription_id, resource_group_name, website_name):
        summary = '\n'
        step_ids = final_status.deployment_step_ids
        if not step_ids or len(step_ids) == 0: return
        steps = {}
        for property in step_ids:
            steps[property.name] = property.value
        # Add the vsts account info
        account_url = 'https://{}.visualstudio.com'.format(quote(account_name))
        if steps['AccountCreated'] == '0':
            summary += "The VSTS account '{}' was updated to handle the continuous delivery.\n".format(account_url)
        elif steps['AccountCreated'] == '1':
            summary += "The VSTS account '{}' was created to handle the continuous delivery.\n".format(account_url)
        # Add the subscription info
        website_url = 'https://portal.azure.com/#resource/subscriptions/{}/resourceGroups/{}/providers/Microsoft.Web/sites/{}/vstscd'.format(
            quote(subscription_id), quote(resource_group_name), quote(website_name))
        summary += 'You can check on the status of the Azure web site deployment here:\n'
        summary += website_url + '\n'
        return summary

    def _skip_update_progress(self, count, total, message):
        return


class _AzureInfo(object):
    def __init__(self):
        self.resource_group_name = None
        self.website_name = None
        self.credentials = None
        self.subscription_id = None
        self.subscription_name = None
        self.tenant_id = None
        self.webapp_location = None


class _RepositoryInfo(object):
    def __init__(self):
        self.url = None
        self.branch = None
        self.git_token = None



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.authentication import (
    BasicAuthentication,
    BasicTokenAuthentication,
    OAuthTokenAuthentication)



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ReferenceLinks(Model):
    """ReferenceLinks.

    :param links: The readonly view of the links.  Because Reference links are
     readonly, we only want to expose them as read only.
    :type links: dict
    """

    _attribute_map = {
        'links': {'key': 'links', 'type': '{object}'},
    }

    def __init__(self, links=None):
        self.links = links



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from .operation_reference import OperationReference


class Operation(OperationReference):
    """Operation.

    :param id: The identifier for this operation.
    :type id: str
    :param status: The current status of the operation.
    :type status: object
    :param url: Url to get the full object.
    :type url: str
    :param _links: The links to other objects related to this object.
    :type _links: :class:`ReferenceLinks <azuretfs.models.ReferenceLinks>`
    :param result_message: The result message which is generally not set.
    :type result_message: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'status', 'type': 'object'},
        'url': {'key': 'url', 'type': 'str'},
        '_links': {'key': '_links', 'type': 'ReferenceLinks'},
        'result_message': {'key': 'resultMessage', 'type': 'str'},
    }

    def __init__(self, id=None, status=None, url=None, _links=None, result_message=None):
        super(Operation, self).__init__(id=id, status=status, url=url)
        self._links = _links
        self.result_message = result_message



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator 1.0.1.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class PipelineConfiguration(Model):
    """PipelineConfiguration.

    :param client_type:
    :type client_type: object
    :param prod_environment_name:
    :type prod_environment_name: str
    :param test_environment_name:
    :type test_environment_name: str
    """

    _attribute_map = {
        'client_type': {'key': 'clientType', 'type': 'object'},
        'prod_environment_name': {'key': 'prodEnvironmentName', 'type': 'str'},
        'test_environment_name': {'key': 'testEnvironmentName', 'type': 'str'},
    }

    def __init__(self, client_type=None, prod_environment_name=None, test_environment_name=None):
        self.client_type = client_type
        self.prod_environment_name = prod_environment_name
        self.test_environment_name = test_environment_name
